package game.network;

import game.packet.S14PacketEntity;
import game.packet.S18PacketEntityTeleport;
import game.packet.S19PacketEntityHeadLook;
import game.packet.S1APacketEntityStatus;
import game.packet.S1BPacketEntityAttach;
import game.packet.S1CPacketEntityMetadata;
import game.packet.S1DPacketEntityEffect;
import game.packet.S1EPacketRemoveEntityEffect;
import game.packet.S20PacketEntityProperties;
import game.packet.S27PacketExplosion;
import game.packet.S28PacketEffect;
import game.packet.S29PacketSoundEffect;
import game.packet.S2APacketParticles;
import game.packet.S2BPacketChangeGameState;
import game.packet.S2CPacketSpawnGlobalEntity;
import game.packet.S2DPacketOpenWindow;
import game.packet.S2EPacketCloseWindow;
import game.packet.S2FPacketSetSlot;
import game.packet.S30PacketWindowItems;
import game.packet.S31PacketWindowProperty;
import game.packet.S32PacketConfirmTransaction;
import game.packet.S33PacketUpdateSign;
import game.packet.S35PacketUpdateTileEntity;
import game.packet.S36PacketSignEditorOpen;
import game.packet.S38PacketPlayerListItem;
import game.packet.S39PacketPlayerAbilities;
import game.packet.S3APacketTabComplete;
import game.packet.S43PacketUpdateEntityNBT;
import game.packet.SPacketAnimation;
import game.packet.SPacketBiomes;
import game.packet.SPacketBlockAction;
import game.packet.SPacketBlockBreakAnim;
import game.packet.SPacketBlockChange;
import game.packet.SPacketCamera;
import game.packet.SPacketCharacterList;
import game.packet.SPacketChunkData;
import game.packet.SPacketCollectItem;
import game.packet.SPacketDestroyEntities;
import game.packet.SPacketDimensionName;
import game.packet.SPacketDisconnect;
import game.packet.SPacketEntityEquipment;
import game.packet.SPacketEntityVelocity;
import game.packet.SPacketHeldItemChange;
import game.packet.SPacketJoinGame;
import game.packet.SPacketKeepAlive;
import game.packet.SPacketLoading;
import game.packet.SPacketMapChunkBulk;
import game.packet.SPacketMessage;
import game.packet.SPacketMultiBlockChange;
import game.packet.SPacketPlayerPosLook;
import game.packet.SPacketRespawn;
import game.packet.SPacketServerTick;
import game.packet.SPacketSetExperience;
import game.packet.SPacketSkin;
import game.packet.SPacketSpawnMob;
import game.packet.SPacketSpawnObject;
import game.packet.SPacketSpawnPlayer;
import game.packet.SPacketTimeUpdate;
import game.packet.SPacketTrades;
import game.packet.SPacketUpdateHealth;
import game.packet.SPacketWorld;

public interface IClientPlayer {
    void addToSendQueue(Packet packet);
    
	void handleJoinGame(SPacketJoinGame packetIn);

	/**
	 * Spawns an instance of the objecttype indicated by the packet and sets its position and momentum
	 */
	void handleSpawnObject(SPacketSpawnObject packetIn);

	/**
	 * Handles globally visible entities. Used in vanilla for lightning bolts
	 */
	void handleSpawnGlobalEntity(S2CPacketSpawnGlobalEntity packetIn);

	/**
	 * Sets the velocity of the specified entity to the specified value
	 */
	void handleEntityVelocity(SPacketEntityVelocity packetIn);

	/**
	 * Invoked when the server registers new proximate objects in your watchlist or when objects in your watchlist have
	 * changed -> Registers any changes locally
	 */
	void handleEntityMetadata(S1CPacketEntityMetadata packetIn);

	/**
	 * Handles the creation of a nearby player entity, sets the position and held item
	 */
	void handleSpawnPlayer(SPacketSpawnPlayer packetIn);

	/**
	 * Updates an entity's position and rotation as specified by the packet
	 */
	void handleEntityTeleport(S18PacketEntityTeleport packetIn);

	/**
	 * Updates which hotbar slot of the player is currently selected
	 */
	void handleHeldItemChange(SPacketHeldItemChange packetIn);

	/**
	 * Updates the specified entity's position by the specified relative moment and absolute rotation. Note that
	 * subclassing of the packet allows for the specification of a subset of this data (e.g. only rel. position, abs.
	 * rotation or both).
	 */
	void handleEntityMovement(S14PacketEntity packetIn);

	/**
	 * Updates the direction in which the specified entity is looking, normally this head rotation is independent of the
	 * rotation of the entity itself
	 */
	void handleEntityHeadLook(S19PacketEntityHeadLook packetIn);

	/**
	 * Locally eliminates the entities. Invoked by the server when the items are in fact destroyed, or the player is no
	 * longer registered as required to monitor them. The latter  happens when distance between the player and item
	 * increases beyond a certain treshold (typically the viewing distance)
	 */
	void handleDestroyEntities(SPacketDestroyEntities packetIn);

	/**
	 * Handles changes in player positioning and rotation such as when travelling to a new dimension, (re)spawning,
	 * mounting horses etc. Seems to immediately reply to the server with the clients post-processing perspective on the
	 * player positioning
	 */
	void handlePlayerPosLook(SPacketPlayerPosLook packetIn);

	/**
	 * Received from the servers PlayerManager if between 1 and 64 blocks in a chunk are changed. If only one block
	 * requires an update, the server sends S23PacketBlockChange and if 64 or more blocks are changed, the server sends
	 * S21PacketChunkData
	 */
	void handleMultiBlockChange(SPacketMultiBlockChange packetIn);

	/**
	 * Updates the specified chunk with the supplied data, marks it for re-rendering and lighting recalculation
	 */
	void handleChunkData(SPacketChunkData packetIn);

	void handleBiomes(SPacketBiomes packetIn);

	/**
	 * Updates the block and metadata and generates a blockupdate (and notify the clients)
	 */
	void handleBlockChange(SPacketBlockChange packetIn);

	/**
	 * Closes the network channel
	 */
	void handleDisconnect(SPacketDisconnect packetIn);

	void handleCollectItem(SPacketCollectItem packetIn);

	/**
	 * Prints a chatmessage in the chat GUI
	 */
	void handleMessage(SPacketMessage packetIn);

	void handleLoading(SPacketLoading packet);

	/**
	 * Renders a specified animation: Waking up a player, a living entity swinging its currently held item, being hurt
	 * or receiving a critical hit by normal or magical means
	 */
	void handleAnimation(SPacketAnimation packetIn);

	/**
	 * Spawns the mob entity at the specified location, with the specified rotation, momentum and type. Updates the
	 * entities Datawatchers with the entity metadata specified in the packet
	 */
	void handleSpawnMob(SPacketSpawnMob packetIn);

	void handleTimeUpdate(SPacketTimeUpdate packetIn);

	void handleServerTick(SPacketServerTick packet);

	void handleEntityAttach(S1BPacketEntityAttach packetIn);

	/**
	 * Invokes the entities' handleUpdateHealth method which is implemented in LivingBase (hurt/death),
	 * MinecartMobSpawner (spawn delay), FireworkRocket & MinecartTNT (explosion), IronGolem (throwing,...), Witch
	 * (spawn particles), Zombie (villager transformation), Animal (breeding mode particles), Horse (breeding/smoke
	 * particles), Sheep (...), Tameable (...), Villager (particles for breeding mode, angry and happy), Wolf (...)
	 */
	void handleEntityStatus(S1APacketEntityStatus packetIn);

	void handleUpdateHealth(SPacketUpdateHealth packetIn);

	void handleSetExperience(SPacketSetExperience packetIn);

	void handleRespawn(SPacketRespawn packetIn);

	/**
	 * Initiates a new explosion (sound, particles, drop spawn) for the affected blocks indicated by the packet.
	 */
	void handleExplosion(S27PacketExplosion packetIn);

	/**
	 * Displays a GUI by ID. In order starting from id 0: Chest, Workbench, Furnace, Dispenser, Enchanting table,
	 * Brewing stand, Villager merchant, Beacon, Anvil, Hopper, Dropper, Horse
	 */
	void handleOpenWindow(S2DPacketOpenWindow packetIn);

	/**
	 * Handles pickin up an ItemStack or dropping one in your inventory or an open container
	 */
	void handleSetSlot(S2FPacketSetSlot packetIn);

	/**
	 * Verifies that the server and client are synchronized with respect to the inventory/container opened by the player
	 * and confirms if it is the case.
	 */
	void handleConfirmTransaction(S32PacketConfirmTransaction packetIn);

	/**
	 * Handles the placement of a specified ItemStack in a specified container/inventory slot
	 */
	void handleWindowItems(S30PacketWindowItems packetIn);

	/**
	 * Creates a sign in the specified location if it didn't exist and opens the GUI to edit its text
	 */
	void handleSignEditorOpen(S36PacketSignEditorOpen packetIn);

	/**
	 * Updates a specified sign with the specified text lines
	 */
	void handleUpdateSign(S33PacketUpdateSign packetIn);

	/**
	 * Updates the NBTTagCompound metadata of instances of the following entitytypes: Mob spawners, command blocks,
	 * beacons, skulls, flowerpot
	 */
	void handleUpdateTileEntity(S35PacketUpdateTileEntity packetIn);

	/**
	 * Sets the progressbar of the opened window to the specified value
	 */
	void handleWindowProperty(S31PacketWindowProperty packetIn);

	void handleEntityEquipment(SPacketEntityEquipment packetIn);

	/**
	 * Resets the ItemStack held in hand and closes the window that is opened
	 */
	void handleCloseWindow(S2EPacketCloseWindow packetIn);

	/**
	 * Triggers Block.onBlockEventReceived, which is implemented in BlockPistonBase for extension/retraction, BlockNote
	 * for setting the instrument (including audiovisual feedback) and in BlockContainer to set the number of players
	 * accessing a (Ender)Chest
	 */
	void handleBlockAction(SPacketBlockAction packetIn);

	/**
	 * Updates all registered IWorldAccess instances with destroyBlockInWorldPartially
	 */
	void handleBlockBreakAnim(SPacketBlockBreakAnim packetIn);

	void handleMapChunkBulk(SPacketMapChunkBulk packetIn);

	void handleChangeGameState(S2BPacketChangeGameState packetIn);

	void handleEffect(S28PacketEffect packetIn);

	void handleEntityEffect(S1DPacketEntityEffect packetIn);

	void handleCamera(SPacketCamera packetIn);

	void handleRemoveEntityEffect(S1EPacketRemoveEntityEffect packetIn);

	void handlePlayerListItem(S38PacketPlayerListItem packetIn);

	void handleCharacterList(SPacketCharacterList packet);

	void handleKeepAlive(SPacketKeepAlive packetIn);

	void handlePlayerAbilities(S39PacketPlayerAbilities packetIn);

	/**
	 * Displays the available command-completion options the server knows of
	 */
	void handleTabComplete(S3APacketTabComplete packetIn);

	void handleSoundEffect(S29PacketSoundEffect packetIn);

	void handleEntityNBT(S43PacketUpdateEntityNBT packetIn);

	/**
	 * Spawns a specified number of particles at the specified location with a randomized displacement according to
	 * specified bounds
	 */
	void handleParticles(S2APacketParticles packetIn);

	/**
	 * Updates en entity's attributes and their respective modifiers, which are used for speed bonusses (player
	 * sprinting, animals fleeing, baby speed), weapon/tool attackDamage, hostiles followRange randomization, zombie
	 * maxHealth and knockback resistance as well as reinforcement spawning chance.
	 */
	void handleEntityProperties(S20PacketEntityProperties packetIn);

	void handleSkin(SPacketSkin packetIn);

	void handleTrades(SPacketTrades packetIn);

	void handleWorld(SPacketWorld packetIn);

	void handleDimName(SPacketDimensionName packetIn);

}